@@ -12,13 +12,14 @@ Python libraries, though it does make heavy use of external packaging commands.
 Visit <https://github.com/datalad/datalad-installer> for more information.
 """
 
-__version__ = "0.1.0"
+__version__ = "0.2.0"
 __author__ = "The DataLad Team and Contributors"
 __author_email__ = "team@datalad.org"
 __license__ = "MIT"
 __url__ = "https://github.com/datalad/datalad-installer"
 
 from abc import ABC, abstractmethod
+from enum import Enum
 from functools import total_ordering
 from getopt import GetoptError, getopt
 import json
@@ -52,6 +53,18 @@ from zipfile import ZipFile
 
 log = logging.getLogger("datalad_installer")
 
+SYSTEM = platform.system()
+ON_LINUX = SYSTEM == "Linux"
+ON_MACOS = SYSTEM == "Darwin"
+ON_WINDOWS = SYSTEM == "Windows"
+ON_POSIX = ON_LINUX or ON_MACOS
+
+
+class SudoConfirm(Enum):
+    ASK = "ask"
+    ERROR = "error"
+    OK = "ok"
+
 
 def parse_log_level(level: str) -> int:
     """
@@ -197,14 +210,14 @@ class Option:
         if self.is_flag:
             namespace[self.dest] = True
         else:
+            if self.choices is not None and argument not in self.choices:
+                raise UsageError(
+                    f"Invalid choice for {self.option_name} option: {argument!r}"
+                )
             if self.converter is None:
                 value = argument
             else:
                 value = self.converter(argument)
-            if self.choices is not None and value not in self.choices:
-                raise UsageError(
-                    f"Invalid choice for {self.option_name} option: {value!r}"
-                )
             if self.multiple:
                 namespace.setdefault(self.dest, []).append(value)
             else:
@@ -422,6 +435,26 @@ class CondaInstance(NamedTuple):
     #: The name of the environment (`None` for the base environment)
     name: Optional[str]
 
+    @property
+    def conda_exe(self) -> Path:
+        """ The path to the Conda executable """
+        if ON_WINDOWS:
+            return self.basepath / "Scripts" / "conda.exe"
+        else:
+            return self.basepath / "bin" / "conda"
+
+    @property
+    def bindir(self) -> Path:
+        """
+        The directory in which command-line programs provided by packages are
+        installed
+        """
+        dirname = "Scripts" if ON_WINDOWS else "bin"
+        if self.name is None:
+            return self.basepath / dirname
+        else:
+            return self.basepath / "envs" / self.name / dirname
+
 
 #: A list of command names and the paths at which they are located
 CommandList = List[Tuple[str, Path]]
@@ -459,11 +492,19 @@ class DataladInstaller:
                     " given file; can be given multiple times"
                 ),
             ),
+            Option(
+                "--sudo",
+                choices=[v.value for v in SudoConfirm],
+                converter=SudoConfirm,
+                help="How to handle sudo commands [default: ask]",
+            ),
         ],
     )
 
     def __init__(
-        self, env_write_files: Optional[List[Union[str, os.PathLike]]] = None
+        self,
+        env_write_files: Optional[List[Union[str, os.PathLike]]] = None,
+        sudo_confirm: SudoConfirm = SudoConfirm.ASK,
     ) -> None:
         #: A list of files to which to write ``PATH`` modifications and related
         #: shell commands
@@ -472,6 +513,7 @@ class DataladInstaller:
             self.env_write_files = []
         else:
             self.env_write_files = [Path(p) for p in env_write_files]
+        self.sudo_confirm: SudoConfirm = sudo_confirm
         #: The default installers to fall back on for the "auto" installation
         #: method
         self.installer_stack: List["Installer"] = [
@@ -517,6 +559,22 @@ class DataladInstaller:
             log.info("Writing environment modifications to %s", fpath)
             self.env_write_files.append(Path(fpath))
 
+    def sudo(self, *args: Any, **kwargs: Any) -> subprocess.CompletedProcess:
+        arglist = [str(a) for a in args]
+        cmd = " ".join(map(shlex.quote, arglist))
+        if self.sudo_confirm is SudoConfirm.ERROR:
+            log.error("Not running sudo command: %s", cmd)
+            sys.exit(1)
+        elif self.sudo_confirm is SudoConfirm.ASK:
+            print("About to run the following sudo command:")
+            print(f"    {cmd}")
+            yan = ask("Proceed?", ["y", "a", "n"])
+            if yan == "n":
+                sys.exit(0)
+            elif yan == "a":
+                self.sudo_confirm = SudoConfirm.OK
+        return runcmd("sudo", *args, **kwargs)
+
     @classmethod
     def parse_args(cls, args: List[str]) -> Union[Immediate, ParsedArgs]:
         """
@@ -593,14 +651,13 @@ class DataladInstaller:
         if global_opts.get("env_write_file"):
             self.env_write_files.extend(global_opts["env_write_file"])
         self.ensure_env_write_file()
+        if global_opts.get("sudo"):
+            self.sudo_confirm = global_opts["sudo"]
         for cr in components:
             self.addcomponent(name=cr.name, **cr.kwargs)
         ok = True
         for name, path in self.new_commands:
             log.info("%s is now installed at %s", name, path)
-            if path.name != name:
-                log.error("Program does not have expected name!")
-                ok = False
             if not os.access(path, os.X_OK):
                 log.error("Cannot execute program!")
                 ok = False
@@ -789,18 +846,22 @@ class MinicondaComponent(Component):
             # command line.)
             path.rmdir()
         log.info("Path: %s", path)
-        log.info("Batch: %s", batch)
+        if ON_WINDOWS:
+            log.info("Batch: True")
+        else:
+            log.info("Batch: %s", batch)
         log.info("Spec: %s", spec)
         log.info("Extra args: %s", extra_args)
         if kwargs:
             log.warning("Ignoring extra component arguments: %r", kwargs)
-        systype = platform.system()
-        if systype == "Linux":
+        if ON_LINUX:
             miniconda_script = "Miniconda3-latest-Linux-x86_64.sh"
-        elif systype == "Darwin":
+        elif ON_MACOS:
             miniconda_script = "Miniconda3-latest-MacOSX-x86_64.sh"
+        elif ON_WINDOWS:
+            miniconda_script = "Miniconda3-latest-Windows-x86_64.exe"
         else:
-            raise RuntimeError(f"E: Unsupported OS: {systype}")
+            raise RuntimeError(f"E: Unsupported OS: {SYSTEM}")
         log.info("Downloading and running miniconda installer")
         with tempfile.TemporaryDirectory() as tmpdir:
             script_path = os.path.join(tmpdir, miniconda_script)
@@ -814,15 +875,27 @@ class MinicondaComponent(Component):
                 script_path,
             )
             log.info("Installing miniconda in %s", path)
-            args = ["-p", path, "-s"]
-            if batch:
-                args.append("-b")
-            if extra_args is not None:
-                args.extend(extra_args)
-            runcmd("bash", script_path, *args)
-        if spec is not None:
-            runcmd(path / "bin" / "conda", "install", *spec)
+            if ON_WINDOWS:
+                # `path` needs to be absolute when passing it to the installer,
+                # but Path.resolve() is a no-op for non-existent files on
+                # Windows.  Hence, we need to create the directory first.
+                path.mkdir(parents=True, exist_ok=True)
+                cmd = f'start /wait "" {script_path}'
+                if extra_args is not None:
+                    cmd += " ".join(extra_args)
+                cmd += f" /S /D={path.resolve()}"
+                log.info("Running: %s", cmd)
+                subprocess.run(cmd, check=True, shell=True)
+            else:
+                args = ["-p", path, "-s"]
+                if batch:
+                    args.append("-b")
+                if extra_args is not None:
+                    args.extend(extra_args)
+                runcmd("bash", script_path, *args)
         conda_instance = CondaInstance(basepath=path, name=None)
+        if spec is not None:
+            runcmd(conda_instance.conda_exe, "install", *spec)
         self.manager.conda_stack.append(conda_instance)
         self.manager.installer_stack.append(
             CondaInstaller(self.manager, conda_instance)
@@ -879,7 +952,7 @@ class CondaEnvComponent(Component):
         if kwargs:
             log.warning("Ignoring extra component arguments: %r", kwargs)
         conda = self.manager.get_conda()
-        cmd = [conda.basepath / "bin" / "conda", "create", "--name", cname]
+        cmd = [conda.conda_exe, "create", "--name", cname]
         if extra_args is not None:
             cmd.extend(extra_args)
         if spec is not None:
@@ -916,12 +989,12 @@ class NeurodebianComponent(Component):
         log.info("Extra args: %s", extra_args)
         if kwargs:
             log.warning("Ignoring extra component arguments: %r", kwargs)
-        runcmd(
+        self.manager.sudo(
             "apt-get",
             "install",
             "-qy",
             "neurodebian",
-            env=dict(os.environ, DEBIAN_FRONTENV="noninteractive"),
+            env=dict(os.environ, DEBIAN_FRONTEND="noninteractive"),
         )
         runcmd("nd-configurerepo", *(extra_args or []))
 
@@ -1043,7 +1116,13 @@ class Installer(ABC):
                 f"{self.NAME} does not know how to install {component}"
             )
         bindir = self.install_package(package, **kwargs)
-        return [(cmd, bindir / cmd) for cmd in commands]
+        bins = []
+        for cmd in commands:
+            p = bindir / cmd
+            if ON_WINDOWS and p.suffix == "":
+                p = p.with_suffix(".exe")
+            bins.append((cmd, p))
+        return bins
 
     @abstractmethod
     def install_package(self, package: str, **kwargs: Any) -> Path:
@@ -1103,7 +1182,7 @@ class AptInstaller(Installer):
         log.info("Extra args: %s", extra_args)
         if kwargs:
             log.warning("Ignoring extra installer arguments: %r", kwargs)
-        cmd = ["sudo", "apt-get"]
+        cmd = ["apt-get"]
         if build_dep:
             cmd.append("build-dep")
         else:
@@ -1114,7 +1193,7 @@ class AptInstaller(Installer):
             cmd.append(f"{package}={version}")
         else:
             cmd.append(package)
-        runcmd(*cmd)
+        self.manager.sudo(*cmd)
         log.debug("Installed program directory: /usr/bin")
         return Path("/usr/bin")
 
@@ -1123,6 +1202,7 @@ class AptInstaller(Installer):
             raise MethodNotSupportedError("apt-get command not found")
 
 
+@DataladComponent.register_installer
 @GitAnnexComponent.register_installer
 class HomebrewInstaller(Installer):
     """ Installs via brew (Homebrew) """
@@ -1134,6 +1214,7 @@ class HomebrewInstaller(Installer):
     ]
 
     PACKAGES = {
+        "datalad": ("datalad", ["datalad"]),
         "git-annex": ("git-annex", ["git-annex"]),
     }
 
@@ -1196,6 +1277,8 @@ class PipInstaller(Installer):
     def python(self) -> Union[str, Path]:
         if self.venv_path is None:
             return sys.executable
+        elif ON_WINDOWS:
+            return self.venv_path / "Scripts" / "python.exe"
         else:
             return self.venv_path / "bin" / "python"
 
@@ -1233,15 +1316,15 @@ class PipInstaller(Installer):
             )
         )
         runcmd(*cmd)
-        if extra_args is not None and "--user" in extra_args:
-            binpath = Path(
-                readcmd(self.python, "-m", "site", "--user-base").strip(),
-                "bin",
+        user = extra_args is not None and "--user" in extra_args
+        binpath = Path(
+            readcmd(
+                self.python,
+                "-c",
+                "from pip._internal.locations import distutils_scheme;"
+                f" print(distutils_scheme({package!r}, user={user!r})['scripts'], end='')",
             )
-        elif self.venv_path is not None:
-            binpath = self.venv_path / "bin"
-        else:
-            binpath = Path("/usr/local/bin")
+        )
         log.debug("Installed program directory: %s", binpath)
         return binpath
 
@@ -1301,11 +1384,11 @@ class DebURLInstaller(Installer):
         with tempfile.TemporaryDirectory() as tmpdir:
             debpath = os.path.join(tmpdir, f"{package}.deb")
             download_file(url, debpath)
-            cmd = ["sudo", "dpkg", "-i"]
+            cmd = ["dpkg", "-i"]
             if extra_args is not None:
                 cmd.extend(extra_args)
             cmd.append(debpath)
-            runcmd(*cmd)
+            self.manager.sudo(*cmd)
             log.debug("Installed program directory: /usr/bin")
             return Path("/usr/bin")
 
@@ -1345,9 +1428,8 @@ class AutobuildSnapshotInstaller(Installer):
             return install_git_annex_dmg(dmgpath, self.manager)
 
     def assert_supported_system(self) -> None:
-        systype = platform.system()
-        if systype not in ("Linux", "Darwin"):
-            raise MethodNotSupportedError(f"{systype} OS not supported")
+        if not ON_POSIX:
+            raise MethodNotSupportedError(f"{SYSTEM} OS not supported")
 
 
 @GitAnnexComponent.register_installer
@@ -1361,10 +1443,9 @@ class AutobuildInstaller(AutobuildSnapshotInstaller):
         if kwargs:
             log.warning("Ignoring extra installer arguments: %r", kwargs)
         assert package == "git-annex"
-        systype = platform.system()
-        if systype == "Linux":
+        if ON_LINUX:
             binpath = self._install_linux("autobuild/amd64")
-        elif systype == "Darwin":
+        elif ON_MACOS:
             binpath = self._install_macos("autobuild/x86_64-apple-yosemite")
         else:
             raise AssertionError("Method should not be called on unsupported platforms")
@@ -1385,10 +1466,9 @@ class SnapshotInstaller(AutobuildSnapshotInstaller):
         if kwargs:
             log.warning("Ignoring extra installer arguments: %r", kwargs)
         assert package == "git-annex"
-        systype = platform.system()
-        if systype == "Linux":
+        if ON_LINUX:
             binpath = self._install_linux("linux/current")
-        elif systype == "Darwin":
+        elif ON_MACOS:
             binpath = self._install_macos("OSX/current/10.10_Yosemite")
         else:
             raise AssertionError("Method should not be called on unsupported platforms")
@@ -1425,6 +1505,10 @@ class CondaInstaller(Installer):
         extra_args: Optional[List[str]] = None,
         **kwargs: Any,
     ) -> Path:
+        if package == "git-annex" and not ON_LINUX:
+            raise MethodNotSupportedError(
+                "Conda only supports installing git-annex on Linux"
+            )
         log.info("Installing %s via conda", package)
         if self.conda_instance is not None:
             conda = self.conda_instance
@@ -1435,7 +1519,7 @@ class CondaInstaller(Installer):
         log.info("Extra args: %s", extra_args)
         if kwargs:
             log.warning("Ignoring extra installer arguments: %r", kwargs)
-        cmd = [conda.basepath / "bin" / "conda", "install"]
+        cmd = [conda.conda_exe, "install"]
         if conda.name is not None:
             cmd.append("--name")
             cmd.append(conda.name)
@@ -1447,10 +1531,7 @@ class CondaInstaller(Installer):
         else:
             cmd.append(f"{package}={version}")
         runcmd(*cmd)
-        if conda.name is None:
-            binpath = conda.basepath / "bin"
-        else:
-            binpath = conda.basepath / "envs" / conda.name / "bin"
+        binpath = conda.bindir
         log.debug("Installed program directory: %s", binpath)
         return binpath
 
@@ -1481,16 +1562,21 @@ class DataladGitAnnexBuildInstaller(Installer):
         assert package == "git-annex"
         with tempfile.TemporaryDirectory() as tmpdir_:
             tmpdir = Path(tmpdir_)
-            systype = platform.system()
-            if systype == "Linux":
+            if ON_LINUX:
                 self.download_latest_git_annex("ubuntu", tmpdir)
                 (debpath,) = tmpdir.glob("*.deb")
-                runcmd("sudo", "dpkg", "-i", debpath)
+                self.manager.sudo("dpkg", "-i", debpath)
                 binpath = Path("/usr/bin")
-            elif systype == "Darwin":
+            elif ON_MACOS:
                 self.download_latest_git_annex("macos", tmpdir)
                 (dmgpath,) = tmpdir.glob("*.dmg")
                 binpath = install_git_annex_dmg(dmgpath, self.manager)
+            elif ON_WINDOWS:
+                self.download_latest_git_annex("windows", tmpdir)
+                (exepath,) = tmpdir.glob("*.exe")
+                runcmd(exepath, "/S")
+                binpath = Path("C:/Program Files", "Git", "usr", "bin")
+                self.manager.addpath(binpath)
             else:
                 raise AssertionError(
                     "Method should not be called on unsupported platforms"
@@ -1499,9 +1585,8 @@ class DataladGitAnnexBuildInstaller(Installer):
         return binpath
 
     def assert_supported_system(self) -> None:
-        systype = platform.system()
-        if systype not in ("Linux", "Darwin"):
-            raise MethodNotSupportedError(f"{systype} OS not supported")
+        if not (ON_LINUX or ON_MACOS or ON_WINDOWS):
+            raise MethodNotSupportedError(f"{SYSTEM} OS not supported")
 
     @staticmethod
     def download_latest_git_annex(ostype: str, target_dir: Path) -> None:
@@ -1564,6 +1649,56 @@ class DataladGitAnnexBuildInstaller(Installer):
         artifact_path.unlink()
 
 
+@GitAnnexComponent.register_installer
+class DataladPackagesBuildInstaller(Installer):
+    """
+    Installs git-annex via artifacts uploaded to
+    <https://datasets.datalad.org/?dir=/datalad/packages>
+    """
+
+    NAME = "datalad/packages"
+
+    OPTIONS: ClassVar[List[Option]] = []
+
+    PACKAGES = {
+        "git-annex": ("git-annex", ["git-annex"]),
+    }
+
+    def install_package(
+        self, package: str, version: Optional[str] = None, **kwargs: Any
+    ) -> Path:
+        log.info("Installing %s via datalad/packages", package)
+        log.info("Version: %s", version)
+        if kwargs:
+            log.warning("Ignoring extra installer arguments: %r", kwargs)
+        assert package == "git-annex"
+        with tempfile.TemporaryDirectory() as tmpdir_:
+            tmpdir = Path(tmpdir_)
+            if ON_WINDOWS:
+                if version is None:
+                    exefile = "git-annex-installer_latest-snapshot_x64.exe"
+                else:
+                    exefile = f"git-annex-installer_{version}_x64.exe"
+                exepath = tmpdir / exefile
+                download_file(
+                    f"https://datasets.datalad.org/datalad/packages/windows/{exefile}",
+                    exepath,
+                )
+                runcmd(exepath, "/S")
+                binpath = Path("C:/Program Files", "Git", "usr", "bin")
+                self.manager.addpath(binpath)
+            else:
+                raise AssertionError(
+                    "Method should not be called on unsupported platforms"
+                )
+        log.debug("Installed program directory: %s", binpath)
+        return binpath
+
+    def assert_supported_system(self) -> None:
+        if not ON_WINDOWS:
+            raise MethodNotSupportedError(f"{SYSTEM} OS not supported")
+
+
 class MethodNotSupportedError(Exception):
     """
     Raised when an installer's `install()` method is called on an unsupported
@@ -1640,6 +1775,14 @@ def install_git_annex_dmg(
     return annex_bin
 
 
+def ask(prompt: str, choices: List[str]) -> str:
+    full_prompt = f"{prompt} [{'/'.join(choices)}] "
+    while True:
+        answer = input(full_prompt)
+        if answer in choices:
+            return answer
+
+
 def main(argv: Optional[List[str]] = None) -> int:
     with DataladInstaller() as manager:
         return manager.main(argv)
